# Generated by Django 5.2.4 on 2026-01-05 12:10

import dogesec_commons.identity.models
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Identity',
            fields=[
                ('id', dogesec_commons.identity.models.IdentityIDField(default='', max_length=64, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(default=None)),
                ('modified', dogesec_commons.identity.models.AutoUpdatedModifiedField(default=None)),
                ('stix', models.JSONField(default=dict)),
            ],
        ),
    ]
